@@ -15,12 +15,11 @@ from tempest.lib import decorators
 from tempest.lib import exceptions as lib_exc
 
 from neutron_tempest_plugin.api import base
-from neutron_tempest_plugin import config
 
 
 class TagTestJSON(base.BaseAdminNetworkTest):
 
-    required_extensions = ['tag']
+    required_extensions = ['standard-attr-tag']
 
     @classmethod
     def resource_setup(cls):
@@ -101,7 +100,7 @@ class TagSubnetTestJSON(TagTestJSON):
 
     @decorators.attr(type='smoke')
     @decorators.idempotent_id('2805aabf-a94c-4e70-a0b2-9814f06beb03')
-    @utils.requires_ext(extension="tag-ext", service="network")
+    @utils.requires_ext(extension="standard-attr-tag", service="network")
     def test_subnet_tags(self):
         self._test_tag_operations()
 
@@ -117,7 +116,7 @@ class TagPortTestJSON(TagTestJSON):
 
     @decorators.attr(type='smoke')
     @decorators.idempotent_id('c7c44f2c-edb0-4ebd-a386-d37cec155c34')
-    @utils.requires_ext(extension="tag-ext", service="network")
+    @utils.requires_ext(extension="standard-attr-tag", service="network")
     def test_port_tags(self):
         self._test_tag_operations()
 
@@ -134,7 +133,7 @@ class TagSubnetPoolTestJSON(TagTestJSON):
 
     @decorators.attr(type='smoke')
     @decorators.idempotent_id('bdc1c24b-c0b5-4835-953c-8f67dc11edfe')
-    @utils.requires_ext(extension="tag-ext", service="network")
+    @utils.requires_ext(extension="standard-attr-tag", service="network")
     def test_subnetpool_tags(self):
         self._test_tag_operations()
 
@@ -150,7 +149,7 @@ class TagRouterTestJSON(TagTestJSON):
 
     @decorators.attr(type='smoke')
     @decorators.idempotent_id('b898ff92-dc33-4232-8ab9-2c6158c80d28')
-    @utils.requires_ext(extension="tag-ext", service="network")
+    @utils.requires_ext(extension="standard-attr-tag", service="network")
     def test_router_tags(self):
         self._test_tag_operations()
 
@@ -177,9 +176,7 @@ class TagFloatingIpTestJSON(TagTestJSON):
     @classmethod
     @utils.requires_ext(extension="router", service="network")
     def _create_resource(cls):
-        cls.ext_net_id = config.CONF.network.public_network_id
-        floatingip = cls.create_floatingip(cls.ext_net_id)
-        return floatingip['id']
+        return cls.create_floatingip()['id']
 
     @decorators.attr(type='smoke')
     @decorators.idempotent_id('53f6c2bf-e272-4e9e-b9a9-b165eb7be807')
@@ -212,13 +209,7 @@ class TagTrunkTestJSON(TagTestJSON):
     def _create_resource(cls):
         network = cls.create_network()
         parent_port = cls.create_port(network)
-        trunk = cls.client.create_trunk(parent_port['id'], None)
-        return trunk['trunk']['id']
-
-    @classmethod
-    def resource_cleanup(cls):
-        cls.client.delete_trunk(cls.res_id)
-        super(TagTrunkTestJSON, cls).resource_cleanup()
+        return cls.create_trunk(parent_port)['id']
 
     @decorators.attr(type='smoke')
     @decorators.idempotent_id('4c63708b-c4c3-407c-8101-7a9593882f5f')
@@ -229,7 +220,7 @@ class TagTrunkTestJSON(TagTestJSON):
 
 class TagFilterTestJSON(base.BaseAdminNetworkTest):
     credentials = ['primary', 'alt', 'admin']
-    required_extensions = ['tag']
+    required_extensions = ['standard-attr-tag']
 
     @classmethod
     def resource_setup(cls):
@@ -331,7 +322,7 @@ class TagFilterSubnetTestJSON(TagFilterTestJSON):
 
     @decorators.attr(type='smoke')
     @decorators.idempotent_id('dd8f9ba7-bcf6-496f-bead-714bd3daac10')
-    @utils.requires_ext(extension="tag-ext", service="network")
+    @utils.requires_ext(extension="standard-attr-tag", service="network")
     def test_filter_subnet_tags(self):
         self._test_filter_tags()
 
@@ -351,7 +342,7 @@ class TagFilterPortTestJSON(TagFilterTestJSON):
 
     @decorators.attr(type='smoke')
     @decorators.idempotent_id('09c036b8-c8d0-4bee-b776-7f4601512898')
-    @utils.requires_ext(extension="tag-ext", service="network")
+    @utils.requires_ext(extension="standard-attr-tag", service="network")
     def test_filter_port_tags(self):
         self._test_filter_tags()
 
@@ -372,7 +363,7 @@ class TagFilterSubnetpoolTestJSON(TagFilterTestJSON):
 
     @decorators.attr(type='smoke')
     @decorators.idempotent_id('16ae7ad2-55c2-4821-9195-bfd04ab245b7')
-    @utils.requires_ext(extension="tag-ext", service="network")
+    @utils.requires_ext(extension="standard-attr-tag", service="network")
     def test_filter_subnetpool_tags(self):
         self._test_filter_tags()
 
@@ -392,7 +383,7 @@ class TagFilterRouterTestJSON(TagFilterTestJSON):
 
     @decorators.attr(type='smoke')
     @decorators.idempotent_id('cdd3f3ea-073d-4435-a6cb-826a4064193d')
-    @utils.requires_ext(extension="tag-ext", service="network")
+    @utils.requires_ext(extension="standard-attr-tag", service="network")
     def test_filter_router_tags(self):
         self._test_filter_tags()
 
@@ -424,9 +415,7 @@ class TagFilterFloatingIpTestJSON(TagFilterTestJSON):
     @classmethod
     @utils.requires_ext(extension="router", service="network")
     def _create_resource(cls):
-        cls.ext_net_id = config.CONF.network.public_network_id
-        floatingip = cls.create_floatingip(cls.ext_net_id)
-        return floatingip['id']
+        return cls.create_floatingip()['id']
 
     def _list_resource(self, filters):
         res = self.client.list_floatingips(**filters)
@@ -467,14 +456,7 @@ class TagFilterTrunkTestJSON(TagFilterTestJSON):
     def _create_resource(cls):
         network = cls.create_network()
         parent_port = cls.create_port(network)
-        trunk = cls.client.create_trunk(parent_port['id'], None)
-        return trunk['trunk']['id']
-
-    @classmethod
-    def resource_cleanup(cls):
-        for res_id in cls.res_ids:
-            cls.client.delete_trunk(res_id)
-        super(TagFilterTrunkTestJSON, cls).resource_cleanup()
+        return cls.create_trunk(parent_port)['id']
 
     def _list_resource(self, filters):
         res = self.client.list_trunks(**filters)
@@ -489,7 +471,7 @@ class TagFilterTrunkTestJSON(TagFilterTestJSON):
 
 class UpdateTagsTest(base.BaseAdminNetworkTest):
 
-    required_extensions = ['tag']
+    required_extensions = ['standard-attr-tag']
 
     def _get_and_compare_tags(self, tags, res_id):
         # nothing specific about networks here, just a resource that is
